#ifndef __PLAYER_H__
#define __PLAYER_H__

#include <boost/shared_ptr.hpp> 

#include "Sprite.h"
#include "Types.h"
#include "Engine.h"


class Player {
public:
    explicit Player(double x, double y, size_t level_width);

    void SetSprites(SpritePtr left, SpritePtr right, SpritePtr stop);
    void Update(double dt);
    void Draw() const;

    double GetX() const { return m_x; }
    double GetY() const { return m_y; }
    double GetDefaultYVelocity()     const { return DefaultYVelocity; }
    double GetDefaultYAcceleration() const { return DefaultYAcceleration; }

    void Run()               { m_running_factor = 2.0; }
    void StopRunning()       { m_running_factor = 1.0; }
    void GoLeft()            { m_vx -= 4.0; m_state=PS::GoLeft; }
    void GoRight()           { m_vx += 4.0; m_state=PS::GoRight; }
    void StopLeft()          { m_vx += 4.0; m_state=PS::Stand; }
    void StopRight()         { m_vx -= 4.0; m_state=PS::Stand; }
    void ForbidGoingLeft()   { m_can_go_left = false; }
    void ForbidGoingRight()  { m_can_go_right = false; }
    void Fall()              { m_vy =  0.0; m_is_on_ground = false; }
    void Jump(double y_velocity = DefaultYVelocity);
    void AllowToJump()       { m_jump_allowed = true; }
    void ForbidToJump()      { m_jump_allowed = false; }
    void SetDefaultMoving()  { m_is_on_ground = false; m_can_go_right = m_can_go_left = true;}
    void PlayerOnGround() {
        m_is_on_ground = m_jump_allowed = true;
        m_vy = 0;
    }

    double GetNextXPosition(double dt) const { return m_x + m_vx * dt * m_running_factor; }
    double GetNextYPosition(double dt) const { return m_y + (m_vy + m_ay * dt) * dt; }

    bool MoveMap() {
        const size_t screen_tiles_count = Engine::Get().Renderer()->GetHorizontalTilesOnScreenCount();
        const size_t half_screen_tiles_count = screen_tiles_count/2;
        return 
            m_x >= m_max_x_pos 
            && 
            m_x < (m_level_width - 1) - half_screen_tiles_count;
    }


private:
    enum { DefaultYVelocity = 20, DefaultYAcceleration=-60 };

    PS::PlayerState m_state;  // stan, w którym znajduje się postać
    SpritePtr m_left;         // animacja, kiedy postać idzie w lewo
    SpritePtr m_right;        // animacja, kiedy postać idzie w prawo
    SpritePtr m_stop;         // animacja, kiedy postać stoi
    double m_x;               // położenie postaci na osi odciętych
    double m_y;               // położenie postaci na osi rzędnych
    double m_vx;              // prędkość na osi OX
    double m_vy;              // prędkość gracza w pionie
    double m_ay;              // przyspieszenie gracza w pionie
    double m_running_factor;  // współczynnik biegania. >1.0 => biegnie, <1.0 => spowolnienie
    bool m_jump_allowed;      // czy gracz może skakać (np. jest na podłożu)
    size_t m_level_width;     // szerokość poziomu (w kaflach)
    bool m_is_on_ground;      // czy postać jest na podłożu
    bool m_can_go_left;       // czy postać może iść w lewo
    bool m_can_go_right;      // czy postać może iść w prawo
    double m_max_x_pos;
};

typedef boost::shared_ptr<Player> PlayerPtr;

#endif
